#include <iostream>
#include <vector>

struct IncompleteLogError {
  std::string output;
};

struct LogEntry {
  std::string input;
  std::string output;
};

template<class T> class LogIterator {
  private:
    T &t;
    typename T::iterator i;

  public:
      LogIterator(T &t): t(t), i(t.begin()) { }

      LogIterator operator ++ (int) {
        LogIterator res(*this);
        ++i;
        return res;
      }

      typename std::string &operator * () const {
        if(i == t.end()) {
          throw IncompleteLogError();
        }

        return i->input;
      }

      void output(const std::string &s) {
        if(i == t.end()) {
          t.push_back(LogEntry{"", s});

          throw IncompleteLogError{s};
        }

        // could also put some sanity checks here
        ++i;
      }
};

template<class T> LogIterator<T> make_logIterator(T &t) {
  return LogIterator<T>(t);
}

class BusinessObject {
  private:
    std::vector<LogEntry> eventLog;

    template<class I> void logic(I &i) {
      if(*i++ == "Enter lair" && *i++ == "Aim Lazor" && *i++ == "Activate") {
        i.output("Heatup");

        if(*i++ == "Muahahaha" && *i++ == "???" && *i++ == "Profit!") {
          std::cout << "World domination!" << std::endl;
        }
      }
    }

  public:
    std::string input(const std::string &s) {
      eventLog.push_back(LogEntry{s, ""});

      auto i = make_logIterator(eventLog);

      try {
        logic(i);
        return "";
      } catch(IncompleteLogError e) {
        return e.output;
      }
    }

};

int main(void) {
  BusinessObject bo;
  bo.input("Enter lair");
  bo.input("Aim Lazor");
  auto r = bo.input("Activate");
  if(r == "Heatup") {
    bo.input("Muahahaha");
    bo.input("???");
    bo.input("Profit!");
  }
}
